%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%   FIGURE 3                                                             % 
%                                                                        %
%   "Expectations formation, sticky prices, and the ZLB"                 %                                    
%                                                                        %
%   Journal of Money, Credit and Banking                                 %
%   Betsy Bersson, Patrick Huertgen and Matthias O. Paustian             %
%                                                                        % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;clear all;clc;
dir = pwd;
cd([dir,'\aux_codes\figure_3']);
return;
%----------------------------------------------------------------
% Run loop over dynare file to produce output from reversal model
%----------------------------------------------------------------

%dynare_path

% delay liftoff for the following periods
T_loop = [5:8];

% periods to save
T=20;
    
for j = 1:length(T_loop)

    delay = T_loop(j);
    
    write_file(delay)
    
    dynare reversal_delay.mod
    
    y{j}  = oo_.endo_simul(strmatch('x',M_.endo_names,'exact'),M_.maximum_lag+1:M_.maximum_lag+T);
    p{j}  = oo_.endo_simul(strmatch('pi_ann',M_.endo_names,'exact'),M_.maximum_lag+1:M_.maximum_lag+T);
    ii{j} = oo_.endo_simul(strmatch('i_ann',M_.endo_names,'exact'),M_.maximum_lag+1:M_.maximum_lag+T);
    tr{j} = oo_.endo_simul(strmatch('i_taylor',M_.endo_names,'exact'),M_.maximum_lag+1:M_.maximum_lag+T);

end


%----------------------------------------------------------------
% Plot Output
%----------------------------------------------------------------
close all;

T = 20;

%% Plot results
cmap = autumn(6); %,'Color',cmap(i,:)
k_linestyles = [{'-.','--',':','-'}];
k_linewidths = [{3,3,3,3,3}];

FS = 20;

figure(1)
for i = 1: 3
    plot(y{i+1},'Color','r','LineStyle',k_linestyles{i},'Linewidth',k_linewidths{i}); axis tight; hold on;
end
plot(y{1},'-b','LineWidth',2); axis tight; hold off;
h = legend('delay 1','delay 2','delay 3','baseline','Location','SouthEast');set(h,'FontSize',FS);legend('boxoff');
h = title('output gap');set(h,'FontSize',FS);
xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print([dir,'\reversal_output_2'],'-depsc')

figure(2)
plot(p{1}./4,'-b','LineWidth',2); axis tight; hold on;
for i = 1: 3
    plot(p{i+1}./4,'Color','r','LineStyle',k_linestyles{i},'Linewidth',k_linewidths{i}); axis tight; hold on;
end
h = title('inflation');set(h,'FontSize',FS);
xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print([dir,'\reversal_infl_2'],'-depsc')

figure(3)
plot((ii{1})./4,'-b','LineWidth',2); axis tight; hold on;
for i = 1: 3
    plot((ii{i+1})./4,'Color','r','LineStyle',k_linestyles{i},'Linewidth',k_linewidths{i}); axis tight; hold on;
end
h = title('interest rate');set(h,'FontSize',FS);
xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print([dir,'\reversal_intrate_2'],'-depsc')

pause(2)
cd(dir);