%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%   FIGURE 5                                                             % 
%                                                                        %
%   "Expectations formation, sticky prices, and the ZLB"                 %                                    
%                                                                        %
%   Journal of Money, Credit and Banking                                 %
%   Betsy Bersson, Patrick Huertgen and Matthias O. Paustian             %
%                                                                        % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all; clc; close all;
% -------------------------------------------------------------------------
% Here we solve the forward looking equations with a fiscal multiplier with
% level-k thinking
% Employ assumption of constant values during the expansion. All future
% values take the value that is calculated in period 1. 
% -------------------------------------------------------------------------

% global paremeters 
beta     = 0.99;
varphi   = 0.85;
kappa    = (1-beta*varphi)*(1-varphi)/(varphi);
phi_pi   = 1.5;
phi_y    = 0.5;
omegainv = 1/2;
sigma    = 2;
svar     = 0.2;
gbar     = 0.01;
%prob     = 5/6; 
prob     = 0.86;

damp    = 0 %0.98;

probVEC = [.8:.0002:.833,.839:.0002:.9];
% probVEC2 = [.8:.01:.9]

big_m = zeros(6,length(probVEC));
big_p = zeros(6,length(probVEC));


for j = 1:length(probVEC)
    
    prob = probVEC(j)


    nu = omegainv;
    s  = svar;

    delta = sigma*(1-prob)*(1-beta*prob)-kappa*(sigma + nu*(1-s))*prob;

    RE_multiplier = sigma*((1-prob)*(1-beta*prob)-kappa*prob)/delta;

    % important params
    k = 500;   %number of levels 

    %initialize matrices
    c = zeros(1,k+1);   
    p = zeros(1,k+1);
    y = zeros(1,k+1);
    m = zeros(1,k+1);
    g = gbar;

    % compute level k experiment
    % loop through k levels - first column is level-0
    for level = 2:k+1

            c(level) = (1-damp)*(prob/(1-prob*beta)*((1-beta)*c(level-1)-inv(sigma)*(0-p(level-1)))) + damp*c(level-1);

            y(level) = (1-svar)*c(level) + svar*g;       

            p(level) =  (1-damp)*(kappa*( sigma*c(level) + omegainv*y(level)) + ...
                        prob/(1-prob*beta*varphi)*(   beta*(1-varphi)*p(level-1) + ...
                        kappa*beta*varphi*( sigma*c(level-1) + omegainv*y(level-1)) )) + damp*p(level-1);

            m(level) = y(level)/(gbar*svar);  %calculates FM and removes level = 0 column

    end

    % RE_solution for vector [pie,y,c]

    cmat = [[1-prob*beta, -kappa*omegainv,-kappa*sigma]; ...
            [0           ,  1             , -(1-s)    ]; ...
            [-1/sigma*prob, 0              ,  (1-prob) ]];

    RE_solution = inv(cmat)*[0;s*g;0];

    RE_y  = RE_solution(2);
    RE_pi = round(RE_solution(1)*400,2)

    multiplier = round(1/s*RE_solution(2)/gbar,2);

    index = [1:5];


    table_m = round(m(index+1),2);
    table_p = round(p(index+1)*400,2)
    
    big_m(:,j) = [multiplier;table_m'];
    big_p(:,j) = [RE_pi;table_p'];

    fprintf('\n ----------------------------------------------- \n');
end

% create plots
cmap = autumn(8);
k_linestyles = [{'-','-.',':','--'}];

level_range=[2:8]
level_range=[2:8]

FS=20;
f = figure(1)
plot(probVEC,big_m(1,:),'*r');axis tight;hold on;
for j = 3:size(big_m,1)
    plot(probVEC,big_m(j,:),'LineStyle',k_linestyles{j-2},'Color',[0,.6,0],'LineWidth',2);axis tight;hold on;
end
hold off;
h = title('Fiscal Multiplier');set(h,'FontSize',FS);
h = legend('Rational Expectations','Level-2','Level-3','Level-4','Level-5','Location','SouthEast');set(h,'FontSize',12);legend('boxoff');
% xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print(f,strcat('MR_multiplier'),'-depsc');

f = figure(2)
plot(probVEC,big_p(1,:),'*r');axis tight;hold on;
for j = 3:size(big_p,1)
    plot(probVEC,big_p(j,:),'LineStyle',k_linestyles{j-2},'Color',[0,.6,0],'LineWidth',2);axis tight;hold on;
end
hold off;
h = title('Inflation');set(h,'FontSize',FS);
h = legend('Rational Expectations','Level-2','Level-3','Level-4','Level-5','Location','SouthEast');set(h,'FontSize',12);legend('boxoff');
% xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print(f,strcat('MR_inflation'),'-depsc');

    
    