%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%   FIGURE 6                                                             % 
%                                                                        %
%   "Expectations formation, sticky prices, and the ZLB"                 %                                    
%                                                                        %
%   Journal of Money, Credit and Banking                                 %
%   Betsy Bersson, Patrick Huertgen and Matthias O. Paustian             %
%                                                                        % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; clc;
addpath([pwd,'\aux_codes\figures_4_6_8\Small Models Workstream\gensys']);

% -------------------------------------------------------------------------
% Solve system of FL equations with interest rate pegged at steady state
% for Tpeg periods. Then in period Tpeg+1, the interest rate drops to ipeg.
% Then the interest rate returns to the Taylor rule. Plot inflation,
% output, and int rate in their own plot for different levels and with the
% REE solution. 
% -------------------------------------------------------------------------

global beta sigma phi_pi phi_y varphi zlb T Tpeg ipeg kappa

% how long to keep ii at 0
Tpeg = 1;
% at T = Tpeg+1, set ii = ipeg. then return to Taylor rule
ipeg = -1 %-.25;

% dynamic variables
T    = 8;    % total number of iterations 
k    = 3;    % number of levels 

% global paremeters
beta   = 0.99;
kappa  = .0255;  
sigma  = 1;
omega  = 0;
phi_pi = 1.5;
phi_y  = 0.5;
options  = optimset('Display','Off','TolFun',1e-12);
x_start  = .5;
varphi = fsolve(inline('P1 - (1-P2*x)*(1-x)/x*(P3 + inv(P4))',4),x_start,options,kappa,beta,omega,sigma);

% important parameters
rho_nr = 0.9;         
zlb = -log(1/beta);     

% initialize matrices
y    = zeros(T+2,k+1);   %rows are quarters, t, columns are levels, K starting at level 0
p    = zeros(T+2,k+1);
ii   = zeros(T+2,k+1);  % taylor rule
r    = zeros(T+1,1);   % natural rate

% create natural rate vector 
r(1,1) = 0;
r(2,1) = 0;      % large shock appears at time T=1
for t = 3:T+2
    r(t,1) = rho_nr*r(t-1,1);
end

% create interest rate shock vector with shock
m = zeros(T+1,1);
% m(2,1) = int_shock;

% -------------------------------------------------------------------------
% Compute Level-K Experiment
% -------------------------------------------------------------------------

for level = 2:k+1
    for index = 1:T+1  %% first row is time t = 0, so start recursion at time t=1

        [sol] = fonc_FL(index,level,y,p,r,m);
        y(index,level)   = sol(1);
        p(index,level)   = sol(2);
        ii(index,level)  = sol(3);
            
    end
end


% -------------------------------------------------------------------------
% Obtain REE Solution
% -------------------------------------------------------------------------
solution_RE = fonc_RE;
y_RE  = solution_RE(1,:)';
p_RE  = solution_RE(2,:)';
ii_RE = solution_RE(3,:)';

% -------------------------------------------------------------------------
% Plot Output
% -------------------------------------------------------------------------
levels =[1,2,3];
TT = 5;

 
FS = 20;
levs = levels+1;
f = figure(1)
plot([y(1:(TT),levs(1))],'LineStyle','-','Color','b','LineWidth',1.5); axis square; axis tight; hold on;
plot([y(1:(TT),levs(2))],'LineStyle','--','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
plot([y(1:(TT),levs(3))],'LineStyle',':','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
%plot([y(1:(TT),levs(4))],'LineStyle','-.','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
h = title('Output Gap');set(h,'FontSize',FS);
xt = get(gca, 'XTick');set(gca, 'FontSize', FS); %change size of tick mark values
print(f,strcat('FL_out_tpeg_',num2str(Tpeg)),'-depsc');
% print(f,strcat('FL_out_tpeg_',num2str(Tpeg)),'-dpdf');

f = figure(2)
plot([p(1:(TT),levs(1))],'LineStyle','-','Color','b','LineWidth',1.5); axis square; axis tight; hold on;
plot([p(1:(TT),levs(2))],'LineStyle','--','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
plot([p(1:(TT),levs(3))],'LineStyle',':','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
h = title('Inflation');set(h,'FontSize',FS);
xt = get(gca, 'XTick'); set(gca, 'FontSize', FS);
print(f,strcat('FL_infl_tpeg_',num2str(Tpeg)),'-depsc');
% print(f,strcat('FL_infl_tpeg_',num2str(Tpeg)),'-dpdf');


f = figure(3)
plot([ii(1:(TT),levs(1))],'LineStyle','-','Color','b','LineWidth',1.5); axis square; axis tight; hold on;
plot([ii(1:(TT),levs(2))],'LineStyle','--','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
plot([ii(1:(TT),levs(3))],'LineStyle',':','Color','r','LineWidth',1.5); axis square; axis tight; hold on;
h = legend(strcat('k=',num2str(levels(1))),strcat('k=',num2str(levels(2))),strcat('k=',num2str(levels(3))),'Location','SouthEast','Orientation','vertical'); 
legend('boxoff');set(h,'FontSize',FS-4);
h = title('Policy Rate');set(h,'FontSize',FS);
xt = get(gca, 'XTick');
set(gca, 'FontSize', FS);
print(f,strcat('FL_intrate_tpeg_',num2str(Tpeg)),'-depsc');
% print(f,strcat('FL_intrate_tpeg_',num2str(Tpeg)),'-dpdf');



