%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%   FIGURE 7                                                             % 
%                                                                        %
%   "Expectations formation, sticky prices, and the ZLB"                 %                                    
%                                                                        %
%   Journal of Money, Credit and Banking                                 %
%   Betsy Bersson, Patrick Huertgen and Matthias O. Paustian             %
%                                                                        % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all; close all;
addpath([pwd,'\aux_codes\figures_1_2_7\level_k']);
addpath([pwd,'\aux_codes\figures_1_1_7\level_k\gensys']);


global damp

damp = 0.98;

beta          = 0.99;  
Tstar         = 100; % upper bound on optimal policy horizon
max_shock_loc = 100; % how far out do anticipated monetary policy shocks go?
Tmax          = 100; % how far out are any IRFs calculated 
ny            = 4; 

k_mat         = [0.05,0.075,0.1];

for loop = 1:length(k_mat)
    
    k = k_mat(loop);
    
    [Yirfs,Rirfs,Rirfs_beliefs] = generate_IRFs_dampen(max_shock_loc,Tmax,k);


    fprintf('\n  Done computing the necessary IRFs to anticipated shocks \n ');

    D = NaN(ny*Tstar,Tstar);
    D_beliefs = NaN(ny*Tstar,Tstar);
    O = zeros(ny*Tstar,ny*Tstar);
    B = NaN(ny*Tstar,1);

    bigbeta = (beta.^([0:Tstar-1]));

    % period loss function
    W                = zeros(ny,ny);
    output_weight    = 1;  
    inflation_weight = 1;
    W(2,2) = output_weight;
    W(3,3) = inflation_weight;

    % load some baseline
    load('baseline.mat');

    % Create D matrix linking observables to anticipated shocks 
    % Create D_beliefs matrix linking observables to anticipated shocks using Rirfs_beliefs 
    % Create B matrix of baseline projection given some shock
    % Create O matrix containing welfare weights
    for  n = 1:ny
         D( (n-1)*Tstar+1 : n*Tstar ,:) = squeeze(Rirfs(n,1:Tstar,:));
         B( (n-1)*Tstar+1 : n*Tstar ,:) = baseline(1:Tstar,n);  % squeeze(baseline(1:Tstar,n,1)); 
         O( (n-1)*Tstar+1 : n*Tstar ,(n-1)*Tstar+1 : n*Tstar) = diag(bigbeta).*W(n,n)*eye(Tstar);  
    end

    % Work out optimal weights on anticipated shocks - (absent ZLB)

    XX = (-(B'*O*D)*pinv(D'*O*D))';

    options = optimset('MaxIter',10^4,'LargeScale','off','Display','Iter','TolFun',10^(-12));

    % Code assumes that interest rate comes first

    A = -D(1:Tstar,:);
    b = B(1:Tstar,1) - (beta-1)*ones(Tstar,1);
    b = B(1:Tstar,1) - (beta-1)*ones(Tstar,1);
    b = B(1:Tstar,1) + log(1/beta)*ones(Tstar,1);

    problem.H      = D'*O*D;        % Symmetric matrix
    problem.f      = (B'*O*D)';     % Vector
    problem.Aineq  = A;             % Matrix for linear inequality constraints
    problem.bineq  = b;             % Vector for linear inequality constraints
    problem.Aeq    = [];            % Matrix for linear equality constraints
    problem.beq    = [];            % Vector for linear equality constraints
    problem.lb	   = [];            % Vector of lower bounds
    problem.ub	   = [];            % Vector of upper bounds
    problem.x0     = ones(Tstar,1); % Initial point for x
    problem.solver = 'quadprog';    % solver 
    problem.options = options;      % Options structure created with optimset

    [X,fval,exitflag,output,lambda] = quadprog(problem);

    if exitflag~=1
        warning('solution not admissable');
    end

    % These are the impulse responses under optimal policy 
    IRF2{loop} = reshape(B+D*X,Tstar,n);

end

IRF_LK = IRF2;

% obtain RE optimal
Figure1baseline
close all;
IRF_RE = IRF2;

NAMES = [{'interest rate','output gap','inflation','real rate gap'}];

% Plot figures
%-------------------------------------------------------------------------
c  = 100;
TT = 30;
index = [1,2,3,4];
figure('Name','Ramsey Policy level-k','NumberTitle','off')
cmap = autumn(length(k_mat)+1);
k_linestyles = [{'-','-.','--'}];
k_markers = [{'diamond','none','none'}];
for j = 1:4
    subplot(2,2,j), 
    for k = 1:length(k_mat) 
        plot(c*IRF_LK{k}(1:TT,index(j)),'Color',[0,.6,0],'LineStyle',k_linestyles{k},'Marker',k_markers{k},'MarkerFaceColor',[0,.6,0],'MarkerSize',2); axis tight; hold on;
    end
    plot(c*baseline(1:TT,index(j)),'-b'); axis tight; hold on;
    plot(c*IRF_RE(1:TT,index(j)),'-or','MarkerFaceColor','r','MarkerSize',2); axis tight; hold on;             
    title(NAMES(index(j)));
end
%legend('level-1','level-2','level-3','baseline','RE optimal');
legend('n=0.05','n=0.075','n=0.1','baseline','RE optimal');
legend boxoff;

print('-depsc','Ramsey_level_1_2_3_dampening');


rmpath([pwd,'\aux_codes\figures_1_5\level_k']);