function [residual, g1, g2, g3] = reversal_delay_static(y, x, params)
%
% Status : Computes static model for Dynare
%
% Inputs : 
%   y         [M_.endo_nbr by 1] double    vector of endogenous variables in declaration order
%   x         [M_.exo_nbr by 1] double     vector of exogenous variables in declaration order
%   params    [M_.param_nbr by 1] double   vector of parameter values in declaration order
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the static model equations 
%                                          in order of declaration of the equations.
%                                          Dynare may prepend or append auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by M_.endo_nbr] double    Jacobian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g2        [M_.endo_nbr by (M_.endo_nbr)^2] double   Hessian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g3        [M_.endo_nbr by (M_.endo_nbr)^3] double   Third derivatives matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

residual = zeros( 7, 1);

%
% Model equations
%

lhs =y(1)*(1+params(1));
rhs =y(1)+params(5)*y(2)+y(1)*params(1)*(1-params(8));
residual(1)= lhs-rhs;
lhs =y(2);
rhs =y(2)*(1-params(7))-1/params(2)*(y(5)/4-y(1)-y(7));
residual(2)= lhs-rhs;
lhs =y(6);
rhs =4*(y(7)+y(1)*params(3)+y(2)*params(4));
residual(3)= lhs-rhs;
lhs =y(3);
rhs =4*y(7);
residual(4)= lhs-rhs;
lhs =y(4);
rhs =y(1)*4;
residual(5)= lhs-rhs;
lhs =y(5);
rhs =4*max(params(6)*x(2)+y(6)*(1-x(2))/4,params(6));
residual(6)= lhs-rhs;
lhs =y(7);
rhs =y(7)*0.9+x(1);
residual(7)= lhs-rhs;
if ~isreal(residual)
  residual = real(residual)+imag(residual).^2;
end
if nargout >= 2,
  g1 = zeros(7, 7);

  %
  % Jacobian matrix
  %

  g1(1,1)=1+params(1)-(1+params(1)*(1-params(8)));
  g1(1,2)=(-params(5));
  g1(2,1)=(-(1/params(2)));
  g1(2,2)=1-(1-params(7));
  g1(2,5)=1/params(2)*0.25;
  g1(2,7)=(-(1/params(2)));
  g1(3,1)=(-(4*params(3)));
  g1(3,2)=(-(4*params(4)));
  g1(3,6)=1;
  g1(3,7)=(-4);
  g1(4,3)=1;
  g1(4,7)=(-4);
  g1(5,1)=(-4);
  g1(5,4)=1;
  g1(6,5)=1;
  g1(6,6)=(-(4*(1-x(2))/4*(params(6)*x(2)+y(6)*(1-x(2))/4>params(6))));
  g1(7,7)=0.09999999999999998;
  if ~isreal(g1)
    g1 = real(g1)+2*imag(g1);
  end
if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],7,49);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],7,343);
end
end
end
end
