function [Yirfs,Rirfs,Rirfs_beliefs] = generate_IRFs(max_shock_loc,Tmax,k)
% -------------------------------------------------------------------------
% Here we solve the model used with Optimal Policy codes and generate IRFs
%
% max_shock_loc: maximum period where the shock can occur
% Tmax: how far out we run the experiment
% k: number of levels used
%
% Rirfs(a,b,c): IRF of a=output,inflation at time b to interest rate shock
% at time c
% Yirfs(a,b,c,d): IFR of a=output,inflation at time b to control shock c at
% time d
% -------------------------------------------------------------------------

%% Create Rirfs matrix: interest rate shock

%initialize matrix
Rirfs = zeros(4,Tmax,max_shock_loc);

% declare shock
nr_shock = 0;
int_shock = 1;

% Solve model for largest shock location
T = max_shock_loc;  % define shock location
[y,p,r] = solve_OP(Tmax,T,k,nr_shock,int_shock);
Rirfs(2,:,T) = y(1:Tmax,k+1)'; 
Rirfs(3,:,T) = p(1:Tmax,k+1)';
Rirfs(1,:,T) = r(1:Tmax,k+1)';  
% calculate expected real interest rate defined as nominal interest rate 
% (level k-1) inflation next period (level k-1).
Rirfs(4,:,T) = r(1:Tmax,k+1)-p(2:Tmax+1,k);

% beliefs
Rirfs_beliefs(2,:,T) = y(1:Tmax,k)'; 
Rirfs_beliefs(3,:,T) = p(1:Tmax,k)';
Rirfs_beliefs(1,:,T) = r(1:Tmax,k)';  
% Rirfs_beliefs(4,:,T) = r(1:Tmax,k)-p(2:Tmax+1,k-1);

% % belief revision
% beliefs_revision(2,:,T) = y(1:Tmax,k)'-y(1:Tmax,k-1)'; 
% beliefs_revision(3,:,T) = p(1:Tmax,k)'-p(1:Tmax,k-1)';
% beliefs_revision(1,:,T) = r(1:Tmax,k)'-r(1:Tmax,k-1)';  
% beliefs_revision(4,:,T) = (r(1:Tmax,k)-p(2:Tmax+1,k-1))-(r(1:Tmax,k-1)-p(2:Tmax+1,k-2));

% fill in Rirfs by shifting IRFs to the left and padding with zeros at the
% end
for n = max_shock_loc-1:-1:1
    Rirfs(1,1:n,n)=Rirfs(1,max_shock_loc-n+1:end,max_shock_loc);
    Rirfs(2,1:n,n)=Rirfs(2,max_shock_loc-n+1:end,max_shock_loc);
    Rirfs(3,1:n,n)=Rirfs(3,max_shock_loc-n+1:end,max_shock_loc);
    Rirfs(4,1:n,n)=Rirfs(4,max_shock_loc-n+1:end,max_shock_loc);
    
    % beliefs
    Rirfs_beliefs(1,1:n,n)=Rirfs_beliefs(1,max_shock_loc-n+1:end,max_shock_loc);
    Rirfs_beliefs(2,1:n,n)=Rirfs_beliefs(2,max_shock_loc-n+1:end,max_shock_loc);
    Rirfs_beliefs(3,1:n,n)=Rirfs_beliefs(3,max_shock_loc-n+1:end,max_shock_loc);
    Rirfs_beliefs(4,1:n,n)=Rirfs(4,max_shock_loc-n+1:end,max_shock_loc);

end

%% Create Yirfs matrix: natural rate shock

%initialize matrix
Yirfs = zeros(4,Tmax,1,max_shock_loc);

% declare shock
nr_shock = 1;
int_shock = 0;

% only apply shock in first period
T = 1;
[y,p,r] = solve_OP(Tmax,T,k,nr_shock,int_shock);
Yirfs(2,:,1,T) = y(1:Tmax,k+1)';  % output 
Yirfs(3,:,1,T) = p(1:Tmax,k+1)';  % inflation
Yirfs(1,:,1,T) = r(1:Tmax,k+1)';  
% only want a 2D matrix -- 4D is unnecessary.
Yirfs = Yirfs(:,:,1,T);

% calculate expected real interest rate defined as nominal interest rate (level k) inflation next period (level k-1).
Yirfs(4,:) = r(1:Tmax,k)-p(2:Tmax+1,k);

end
