function [IRF,P0,N0] = solve_model(A0,A1,A2,A3,T_IRF,div)

% INPUT:   
%          A0,A1,A2,A3            - system matrices see below
%          beta                   - discount factor
%          T_IRF                  - length of periods for impulse responses
% 
% OUTPUT: IRFs                    - impulse responses
%         P0                      - transition matrix in post-terminal regime
%
% 

% System given as
% g0*y(t)=g1*y(t-1)+c+psi*z(t)+pi*eta(t),

[dim,dv]  = size(A3);   % number of variables + shocks

% put system into gensys format to solve for reduced-form transition equation in that regime
g0  = [[A0, -A2];[eye(dim), zeros(dim)]];        % coefficients on current period variables (including expectations of tomorrows variables as separate variables)
g1	= [[A1,zeros(dim)];[zeros(dim),eye(dim)]];   % coefficients on last period variables
c   = zeros(2*dim,1);                            % constant (not present in terminal regime)
psi = [A3;zeros(dim,dv)];                        % coefficients on structural shocks 
pie = [zeros(dim);eye(dim)];                     % coefficients on expectational errors
% div = 1.01;                                     % cutoff between explosive and non-explosive roots

addpath([cd,'\gensys']);
[NA,c,NB,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,c,psi,pie,div);
rmpath([cd,'\gensys']);

% Extract transition matrix (discard expectational variables that are of no interest here)
P0 = NA(1:dim,1:dim);
N0 = NB(1:dim,1:dv);

% max_eig = max(abs(eig(P0)));
% fprintf('\n Maximum eigenvalue of the system = %g \n',max_eig);

% compute impulse responses
% --------------------------
IRF = zeros(T_IRF,dim,dv);

for k = 1 : dv;
    IRFs     = zeros(dim,T_IRF); % Use to store response to specified shocks
    shock    = zeros(dv,1);
    shock(k) = 1;
    IRFs(:,1) = N0*shock;
    i = 2;
    while i <= T_IRF
        IRFs(:,i) = P0*IRFs(:,i-1);
        i = i+1;
    end
    IRF(:,:,k) = IRFs';
end
    