function [Yirfs,Rirfs,Rirfs_Beliefs] = generate_IRFs_damp(max_shock_loc,Tmax,k)
% -------------------------------------------------------------------------
% Here we solve the model used with Optimal Policy codes and generate IRFs
%
% max_shock_loc: maximum period where the shock can occur
% Tmax: how far out we run the experiment
% k: number of levels used
%
% Rirfs(a,b,c): IRF of a=output,inflation at time b to interest rate shock
% at time c
% Yirfs(a,b,c,d): IFR of a=output,inflation at time b to control shock c at
% time d
% -------------------------------------------------------------------------

%% Create Rirfs matrix: interest rate shock

global damp

kk = round(k/(1-damp));    % number of levels 


%initialize matrix
Rirfs = zeros(2,Tmax,max_shock_loc);

% declare shock
nr_shock  = 0;
int_shock = 1;

% loop through each shock location, T 
for T = 1:max_shock_loc
    
    [y,p,r] = solve_OP_damp(Tmax,T,k,nr_shock,int_shock);
    
    Rirfs(2,:,T) = y(2:Tmax+1,kk+1)';  % output
    Rirfs(3,:,T) = p(2:Tmax+1,kk+1)';  % inflation 
    Rirfs(1,:,T) = r(2:Tmax+1,kk+1)';  
    Rirfs(4,:,T) = p(2:Tmax+1,kk+1)'-p(1:Tmax,kk+1)';
    Rirfs(5,:,T) = r(1:Tmax,kk+1)-p(2:Tmax+1,kk);
    
    
end

%% Create Yirfs matrix: natural rate shock
Yirfs = [];


% %initialize matrix
% Yirfs = zeros(3,Tmax,1,max_shock_loc);
% 
% % declare shock
% nr_shock = 1;
% int_shock = 0;
% 
% % only apply shock in first period
% T = 1;
% [y,p,r] = solve_OP_damp(Tmax,T,k,nr_shock,int_shock);
% 
% Yirfs(2,:,1,T) = y(2:Tmax+1,k+1)';  % output 
% Yirfs(3,:,1,T) = p(2:Tmax+1,k+1)';  % inflation
% Yirfs(1,:,1,T) = r(2:Tmax+1,k+1)';  
% % only want a 2D matrix -- 4D is unnecessary.
% Yirfs = Yirfs(:,:,1,T);
% 
% 
end
