clear all; clc; close all;

Tstar                    = 40; % upper bound on duration of ZLB

global UD
UD = 0;  % solve model with no ueber discounting 

[AA0,AA1,AA2,AA3,r,beta] = get_model;
[A0,A1,A2,A3]            = add_anticipated(AA0,AA1,AA2,AA3,Tstar,r);

T_IRF  = Tstar+60;

[IRF,P0,S0] = solve_model(A0,A1,A2,A3,T_IRF,1); 


Rpos  = 1;
shock = -0.0165*1.1;

% shock = -0.0165*1.2;


ZLB   = -log(1/beta)*ones(Tstar,1);
v     = IRF(1:Tstar,Rpos,1)*shock - ZLB;    
M     = squeeze(IRF(1:Tstar,Rpos,2:end));
b     = zeros(Tstar,1);
LB    = zeros(Tstar,1);
X0    = ones(Tstar,1);
options = optimset('MaxIter',10^4,'LargeScale','off','Display','Iter','TolFun',10^(-12),'Algorithm','active-set');

problem.H      = (M+M'); % Symmetric matrix
problem.f      = v';     % Vector
problem.Aineq  = -M;     % Matrix for linear inequality constraints
problem.bineq  = v;      % Vector for linear inequality constraints
problem.Aeq    = [];     % Matrix for linear equality constraints
problem.beq    = [];     % Vector for linear equality constraints
problem.lb	   = LB;     % Vector of lower bounds
problem.ub	   = [];     % Vector of upper bounds
problem.x0     = X0;     % Initial point for x
problem.solver = 'quadprog';  % solver 
problem.options = options;    % Options structure created with optimset

[X,fval,exitflag,output,lambda] = quadprog(problem);

fprintf('\n Duration at ZLB = %g \n',sum(X>10*sqrt(eps)));


NAMES = [{'interest rate','Output','inflation','real rate gap'}];

% Construct responses to joint shock: structural + anticipated
%-------------------------------------------------------------------------
Malpha = squeeze(IRF(1:T_IRF,:,2))*X(1);
for p = 2 : Tstar;
   Malpha = Malpha + squeeze(IRF(1:T_IRF,:,p+1))*X(p);
end
joint_IRF            = squeeze(IRF(:,:,1))*shock;
joint_IRF(1:T_IRF,:) = joint_IRF(1:T_IRF,:) + Malpha; 


% Plot figures
%-------------------------------------------------------------------------
c=100;
index = [1,2,3,4];
figure('Name','Commitment Responses (1)','NumberTitle','off')
for j = 1:4
    subplot(2,2,j), plot(c*joint_IRF(:,j),'-*r'); axis tight;
    title(NAMES(index(j)));
end;

orient landscape
print('-dpdf','Taylor_rule_ZLB');


baseline_fbl = joint_IRF;

save baseline_fbl baseline_fbl