function [y_tr,p_tr,ii_tr] = solve_OP(Tmax, shock_loc, num_levels, nr_shock, int_shock)
% -------------------------------------------------------------------------
% Solve system of FBL equations contained in fonc_OP.m
% -------------------------------------------------------------------------

global beta omega sigma phi_pi phi_y varphi zlb T

% dynamic variables
T    = Tmax;    % total number of iterations 
k    = num_levels;    % number of levels 

% global paremeters
beta   = 0.99;
kappa  = .0255;  % kapval;  
omega = 0;
sigma  = 1;
phi_pi = 0*1.5;
phi_y  = 0*0.5;
options  = optimset('Display','Off','TolFun',1e-12);
x_start  = .5;
varphi = fsolve(inline('P1 - (1-P2*x)*(1-x)/x*(P3 + inv(P4))',4),x_start,options,kappa,beta,omega,sigma);

% important parameters
rho_nr = 0.9;          % shock persistance
zlb = -log(1/beta);      % zlb

% initialize matrices
y_tr    = zeros(T+2,k+1);   %rows are quarters, t, columns are levels, K starting at level 0
p_tr    = zeros(T+2,k+1);
ii_tr   = zeros(T+2,k+1);  % taylor rule
r       = zeros(T+2,1);   % natural rate

% create natural rate vector 
r(1,1) = 0;
r(2,1) = nr_shock;      % large shock appears at time T=1
for t = 3:T+2
    r(t,1) = rho_nr*r(t-1,1);
end

% create interest rate shock vector with shock
m = zeros(T+2,1);
m(shock_loc+1,1) = int_shock;

% -------------------------------------------------------------------------
% Compute Level-K Experiment
% -------------------------------------------------------------------------

for level = 2:k+1
    for index = 2:T+1  %%first row is time t = 0, so start recursion at time t=1

        [sol] = fonc_OP(index,level,y_tr,p_tr,r,m);
        y_tr(index,level)   = sol(1);
        p_tr(index,level)   = sol(2);
        ii_tr(index,level)  = sol(3);
            
    end
end


end
