function [IRF_TAY_1,IRF_TAY_NK] = FBL_ree(shockval,UD,UDP)
addpath('I:\m1elb00\projects\Matthias\level_k\replication_codes\aux_codes\figure_7_table_1\Small Models Workstream');
addpath('I:\m1elb00\projects\Matthias\level_k\replication_codes\aux_codes\figure_7_table_1\Small Models Workstream\gensys');

%-------------------------------------------------------------------------
% Solve the model under rational expectations. Use this output to compare
% with level-k; UD is the ueber discounting value placed on future output
% in the euler equation
%-------------------------------------------------------------------------

%% Calibration and Initialization

pars.UD          = UD;  % ueber discounting value in euler eqn
pars.UDP         = UDP; % ueber discounting value in phillips curve

pars.ALPHA       = 0.66;                      %[1]  Probability of resetting prices [Gali = 2/3; Rotemberg and Woodford = 0.66; Steinsson 0.7]; Gali and Gertler estimate [0.803, 0.866]
                                              %     Implies an average price duration of three quarters
% never used, kappa is calibrated directly in code                                              
                                              
pars.BETA        = 0.99;                      %[2]  Discount factor (as in Gali and Steinsson; Gali and Gertler estimate it to be [0.885, 0.957])
pars.DELTA       = 0.052;                     %[3]  Weight on backward looking component in rule of thumb rule for myopic households
                                              %     Steinsson sets this to 0.52 in order to ensure that the coefficient on x(-1) in the backward looking Philips curve is equal to 
                                              %     KAPPA_1 when SBL = 0 we stick to this as well
pars.OMEGA       = 0.5;                       %[4]  Steinsson sets PSI = 2 (based on Rotember and Woodford estimates of 2.13) implying a value of 1/2; Gali uses 1
pars.RHO_N       = 0.9;                       %[5]  Shock persistence (ad hoc)
pars.SIGMA       = 1;                     %[6]  Interest rate elasticity of demand (Steinson sets SIGMA = 5, Gali sets SIGMA = 1, Rotemberg and Woodford estimate 6.4
                                              %     Crucially, as discussed in Amato and Laubach p. 310, SIGMA < 1 (note their SIGMA is the inverse of ours, hence the adjusted value) 
                                              %     "is necessary for the effrect of past consumption on current marginal utility to be positive, which is the essence of habit formation" 
                                              %     Without taking a stand on whether this is true or not, we note that SIGMA > 1 produces ridiculous results when H > 0 (?)
pars.THETA       = 5;                         %[7]  Number used by Steinsson; Gali uses 6; Rotemberg and Woodford estimate 7.88

%Taylor rule coefficients
%i = VARPHI_LI*i(-1) + VARPHI_LX *x(-1) + VARPHI_LPI*pi(-1) 
%  + VARPHI_R *r_nat + VARPHI_X*x       + VARPHI_PI *pi    
%                    + VARPHI_FX*x(+1)  + VARPHI_FPI*pi(+1) 

pars.VARPHI_R    = 1;                         %[9]  Coefficient on the natural rate
pars.VARPHI_LI   = 0;                         %[10] Coefficient on the lagged policy rate
pars.VARPHI_LX   = 0;                         %[11] Coefficient on the lagged output gap
pars.VARPHI_LPI  = 0;                         %[12] Coefficient on lagged inflation
pars.VARPHI_PI   = 1.00001; %1.5;                       %[13] Coefficient on current inflation
pars.VARPHI_X    = 0; %0.5;                      %[14] Coefficient on the current output gap
pars.VARPHI_FX   = 0;                         %[15] Coefficient on the future output gap
pars.VARPHI_FPI  = 0;                         %[16] Coefficient on future inflation

pars.SHOCK       =  shockval; %-0.015;  %       -0.0165;   %         %[17] Size of the shock (calibrated to yield a 10% fall in the output gap under commitment in the simple model)
pars.T_IRF       = 500;                        %[18] Length of the IRFs to plot

%We'll just check that the model works well with all of them
pars.H           = 0.2;                       %[19] Amato and Laubach consider values of {0, 0.4, 0.8}
pars.SBL         = 0.2;                       %[20] Steinsson considers four different parameter values {0.01, 0.2, 0.7, 0.99}

%Other definitions
vars        = {'x','pi','i','r_nat'};
var_names   = {'Output gap','Inflation','Policy Rate','Natural Rate'};

scaling_SS  = [0  ,0  ,400*log(1/pars.BETA),400*log(1/pars.BETA)];
scaling     = [100,400,400          ,400];

%% Section 1

% Call the main routines
[IRF_TAY_1,Welf_TAY_1] = Generate_IRF('NK_TAYLOR','Taylor',vars,pars);

%% Section 2

%In this section we investigate the implications of "lower for longer" policy

%Start by computing length at ZLB under Taylor rule 
len_1 = numel(IRF_TAY_1(3,abs(IRF_TAY_1(3,:) + log(1/pars.BETA))<10e-10));

%Run simulations extending period at ZLB by 1 to 10 periods
for extra_periods = 1:10
    [IRF_TAY_NK{extra_periods},WELF_TAY_NK(extra_periods)] = Generate_IRF('NK_TAYLOR','Taylor_nbnd',vars,pars,len_1+extra_periods);
end

rmpath('I:\m1elb00\projects\Matthias\level_k\replication_codes\aux_codes\figure_7_table_1\Small Models Workstream');
rmpath('I:\m1elb00\projects\Matthias\level_k\replication_codes\aux_codes\figure_7_table_1\Small Models Workstream\gensys');

end
