function [IRF_RET,Welfare] = Taylor(model,vars,params)

%This function returns the IRFs corresponding to 'model' run under a Taylor rule
%for variables given in vector vars (in corresponding order) and for a
%vector of parameters params

% 1. Obtain linear model matrices, welfare weight matrix (which won't be
% used) and variable numbers corresponding to vars;
%--------------------------------------------------------------------------

%Note the model will be in a form compliant with this piece of code
eval(['[A0,A1,A2,W,VAR_NUMS] = get_model_',model,'(vars,params);']);

% 2. Decode parameter vector
dy = size(A1,1); 

BETA        = params.BETA;     %[1] Discount factor
INIT_SHOCK  = params.SHOCK;    %[2] Size of the shock (calibrated to yield a 10% fall in the output gap under commitment)
RHO_N       = params.RHO_N;    %[3] AR coefficient on the shock  
T_IRF       = params.T_IRF;    %[4] Number of periods for which impulse responses are to be computed

%3. setup constraint matrix S, premultiplies x_t in occasional constra
%-------------------------------------------------
s   = -log(1/BETA);     %The ZLB

% 4. solve model with stacked time approach
% -----------------------------------------

%Assumption 1: Nominal interest rate i comes last
%Assumption 2: Shadow rate comes one but last

loc(1) = dy; 
loc(2) = dy;


SHOCK = [INIT_SHOCK,zeros(1,dy-1)]';
    
%Auxiliary variable definitions
check = 0;
d     = 0;

while ~check && (d < T_IRF)
    d = d +1;
    
    disp(sprintf('Taylor rule solution, trying length at ZLB of periods %2.0f', d))
    
    [Z,P,J] = solve_stacked_shock(A0,A1,A2,d,loc,s,SHOCK,T_IRF); 
    
    IRF_all = reshape(Z,dy,T_IRF);
    
    check_shadow_negative = ~any(IRF_all(dy-1,d+1:end)<s);
%       if check_shadow_negative == 0;
%           fprintf('\n Shadow rate is negative when assumed positive \n');
%       end;
    
    check_actual_negative = ~any(IRF_all(dy,d+1:end)<s);
%       if check_actual_negative == 0;
%           fprintf('\n ZLB violated after exit \n');
%       end;
    
    check_false_negative  = ~any(IRF_all(dy-1,1:d)>s);
%      if check_false_negative == 0;
%          fprintf('\n ZLB imposed when shadow rate is positive \n');
%      end;
    
    check = check_shadow_negative*check_actual_negative*check_false_negative;
end
   
% 7. Now return only the desired IRFs in the desired order
%---------------------
IRF_RET = IRF_all(VAR_NUMS,:); %Choose the relevant rows of the IRFs matrix

Welfare = 0;
for i=1:T_IRF
    %We pick the model variables (excluding the policy instrument)
    Welfare = Welfare + BETA^(i-1)*IRF_all(:,i)'*W*IRF_all(:,i);
end
end
