function IRFs = compute_IRFS_discretion(H1S,H2S,H3S,H4S,F1S,F2S,F3S,F4S,MUS,IRF_length,v0,dy,dx,dv,dmu,T)
                                      

% Will include multipliers on occasional constraints (except when out of terminal regime) in this. 
% To be ordered: y variables, x variables, multipliers.
IRFs = zeros((dy+dx+dmu),IRF_length); 
IRFs(1:dy,1)                  =  H3S(1:dy,:)*v0 + H4S(1:dy,1);
IRFs((dy+1):(dy+dx),1)        =  F3S(1:dx,:)*v0 + F4S(1:dx,1);
IRFs((dy+dx+1):(dy+dx+dmu),1) =  MUS(1:dmu,(dy+dx+1):(dy+dx+dv))*v0 + MUS(1:dmu,(dy+dx+dv+1)); 
% Note columns in this matrix correspond in turn to coefficients on y(-1), x(-1), v and a constant

i = 2;
while i < T + 1
    IRFs(1:dy,i)                  = H1S((dy*(i-1)+1):(dy*i),:)*IRFs(1:dy,(i-1)) + ...
                                    H2S((dy*(i-1)+1):(dy*i),:)*IRFs((dy+1):(dy+dx),(i-1)) + ...
                                    H4S((dy*(i-1)+1):(dy*i),1); % Fills out endogenous variables
               
    IRFs((dy+1):(dy+dx),i)        = F1S((dx*(i-1)+1):(dx*i),:)*IRFs(1:dy,(i-1)) + ...
                                    F2S((dx*(i-1)+1):(dx*i),:)*IRFs((dy+1):(dy+dx),(i-1)) + ...
                                    F4S((dx*(i-1)+1):(dx*i),1); % Fills out controls
                         
    IRFs((dy+dx+1):(dy+dx+dmu),i) = MUS((dmu*(i-1)+1):(dmu*i),1:dy)*IRFs(1:dy,(i-1)) + ...
                                    MUS((dmu*(i-1)+1):(dmu*i),(dy+1):(dy+dx))*IRFs((dy+1):(dy+dx),(i-1)) + ...
                                    MUS((dmu*(i-1)+1):(dmu*i),(dy+dx+dv+1)); % Fills out multipliers
    i = i+1;
end

% Next, fill out these matrices for the case where the ZLB regime is over. 
% Note we haven't any need to store values of the multipliers in the terminal regime. 
while i < IRF_length + 1
    IRFs(1:dy,i)                  = H1S((dy*(T)+1):(dy*(T+1)),:)*IRFs(1:dy,(i-1)) + ...
                                    H2S((dy*(T)+1):(dy*(T+1)),:)*IRFs((dy+1):(dy+dx),(i-1)) + ...
                                    H4S((dy*(T)+1):(dy*(T+1)),1); % Fills out endogenous using terminal regime
                                
    IRFs((dy+1):(dy+dx),i)        = F1S((dx*(T)+1):(dx*(T+1)),:)*IRFs(1:dy,(i-1)) + ...
                                    F2S((dx*(T)+1):(dx*(T+1)),:)*IRFs((dy+1):(dy+dx),(i-1)) + ...
                                    F4S((dx*(T)+1):(dx*(T+1)),1); % Fills out controls
                                
    IRFs((dy+dx+1):(dy+dx+dmu),i) = MUS((dmu*(T)+1):(dmu*(T+1)),1:dy)*IRFs(1:dy,(i-1)) + ...
                                    MUS((dmu*(T)+1):(dmu*(T+1)),(dy+1):(dy+dx))*IRFs((dy+1):(dy+dx),(i-1)) + ...
                                    MUS((dmu*(T)+1):(dmu*(T+1)),(dy+dx+dv+1)); % Fills out multipliers
    i = i+1;
end
