function [A0,A1,A2,A3,A5,A6,W,VAR_NUMS] = get_model_NK_OPT(vars,pars)

ALPHA   = pars.ALPHA;
BETA    = pars.BETA;
OMEGA   = pars.OMEGA;
RHO_N   = pars.RHO_N;
SIGMA   = pars.SIGMA;
THETA   = pars.THETA;

KAPPA   = (1-ALPHA)*(1-ALPHA*BETA)/ALPHA*1/(1+OMEGA*THETA)*(OMEGA+1/SIGMA);

% order of variables in vector y_t
pi      = 1; 
r_nat   = 2;  
x       = 3;  
y       = 4;  
y_nat   = 5;  

% order of variables in vector x_t
i = 1;

% order of variables in vector v_t
epsilon_R  = 1;


% Model is of the form:
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X_{t} + A6*X_{t-1} + A5*V_{t}
 
dy = 5; dx = 1; dv = 1;

VAR_NUMS = zeros(size(vars,2),1);

for j = 1:size(vars,2)
    if vars{j} == 'i'
        VAR_NUMS(j) = dy + i;
    else
        VAR_NUMS(j) = eval(vars{j});
    end
end


A0 = zeros(dy,dy);
A1 = zeros(dy,dy);
A2 = zeros(dy,dy);
A3 = zeros(dy,dx);
A5 = zeros(dy,dv);
A6 = zeros(dy,dx);

%[1] New Keynesian Philips Curve (see also script for details)
%pi - KAPPA*x = BETA*pi(+1);
A0(1,pi)    = 1;
A0(1,x)     = -KAPPA;
A2(1,pi)    = BETA;

%[2] IS Equation
%x - SIGMA* r_nat = SIGMA* pi(+1)+ x(+1) - SIGMA*i;
A0(2,x)     =  1;
A0(2,r_nat) = -SIGMA;
A2(2,pi)    =  SIGMA;
A2(2,x)     =  1;
A3(2,i)     = -SIGMA;

%[3] Natural Rate Persistence
%r_nat = RHO_N*r_nat(-1) + epsilon_R;
A0(3,r_nat) = 1;
A1(3,r_nat) =RHO_N;
A5(3,epsilon_R)  = 1;

%[4] Definition of the Output Gap
%y  - x - y_nat = 0;
A0(4,y)     =  1;
A0(4,x)     = -1;
A0(4,y_nat) = -1;

%[5] Equation Linking the Natural Rate of Output (y_n) and the Natural Rate of Interest (r_n)
%y_nat + SIGMA*r_nat = 0.99*y_nat(+1) ;
A0(5,y_nat) =  1;
A0(5,r_nat) =  SIGMA;
A2(5,y_nat) =  0.99;

% Initialise weight matrix:
W         = zeros(dy); % This is the weight matrix in the loss function
W(pi,pi)  = 1;
W(x,x)    = KAPPA/THETA;