function [A0,A1,A2,W,VAR_NUMS] = get_model_NK_TAYLOR(vars,pars)

ALPHA   = pars.ALPHA;
BETA    = pars.BETA;
OMEGA   = pars.OMEGA;
RHO_N   = pars.RHO_N; 
SIGMA   = pars.SIGMA;
THETA   = pars.THETA;

UD = pars.UD;
UDP = pars.UDP;

%Taylor rule coefficients
VARPHI_R   = pars.VARPHI_R;  %Coefficient on the natural rate
VARPHI_LI  = pars.VARPHI_LI; %Coefficient on the lagged policy rate
VARPHI_LX  = pars.VARPHI_LX; %Coefficient on the lagged output gap
VARPHI_LPI = pars.VARPHI_LPI;%Coefficient on lagged inflation
VARPHI_PI  = pars.VARPHI_PI; %Coefficient on current inflation
VARPHI_X   = pars.VARPHI_X;  %Coefficient on the current output gap
VARPHI_FX  = pars.VARPHI_FX; %Coefficient on the future output gap
VARPHI_FPI = pars.VARPHI_FPI;%Coefficient on future inflation

KAPPA   = 0.0255;  % (1-ALPHA)*(1-ALPHA*BETA)/ALPHA; 

% Order of variables in vector y_t
% Not sure it matters but defining r_nat as the first variable as well
r_nat   = 1;  
pi      = 2; 
x       = 3;  
y       = 4;  
% y_nat   = 5;  
i_shadow = 5;
i        = 6;

dy = i;  %Since the interest rate always comes last this will always work

% Model is of the form:
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1}
 
VAR_NUMS = zeros(size(vars,2),1);

for j = 1:size(vars,2)
    VAR_NUMS(j) = eval(vars{j});
end

A0 = zeros(dy,dy);
A1 = zeros(dy,dy);
A2 = zeros(dy,dy);

%ASSUMPTION: Natural rate equation comes first
%[1] Natural Rate (shock will be added via policy shocks)
%r_nat = RHO_N*r_nat(-1) + epsilon_R;
A0(1,r_nat) = 1;
A1(1,r_nat) = RHO_N;

%[2] New Keynesian Philips Curve (see also script for details)
%pi - KAPPA*x = BETA*pi(+1);
A0(2,pi)    = 1+BETA;
A0(2,x)     = -KAPPA;
A2(2,pi)    = (1-UDP)*BETA;
A1(2,pi)    = 1;

%[3] IS Equation
%x - SIGMA* r_nat + SIGMA*i = SIGMA* pi(+1)+ x(+1);
A0(3,x)     =  1;
A0(3,r_nat) = -SIGMA;
A2(3,pi)    =  SIGMA;
A2(3,x)     =  (1-UD)*1;
A0(3,i)     =  SIGMA;

%[4] Definition of the Output Gap
%y  - x - y_nat = 0;
A0(4,y)     =  1;
A0(4,x)     = -1;
% A0(4,y_nat) = -1;

% %[5] Equation Linking the Natural Rate of Output (y_n) and the Natural Rate of Interest (r_n)
% %y_nat + SIGMA*r_nat = 0.99*y_nat(+1) ;
% A0(5,y_nat) =  1;
% A0(5,r_nat) =  SIGMA;
% A2(5,y_nat) =  1;

%[6] Shadow Rate Definition 
%i_shadow - VARPHI_R*r_nat - VARPHI_PI*pi - VARPHI_X*x = VARPHI_LI*i(-1) + VARPHI_LX*x(-1) + VARPHI_LPI*pi(-1) + VARPHI_FX*x(+1) + VARPHI_FPI*pi(+1) 
A0(5,i_shadow) =  1;
A0(5,r_nat)    = -VARPHI_R;
A0(5,pi)       = -VARPHI_PI;
A0(5,x)        = -VARPHI_X;
A1(5,i)        =  VARPHI_LI;
A1(5,x)        =  VARPHI_LX;
A1(5,pi)       =  VARPHI_LPI;
A2(5,x)        =  VARPHI_FX;
A2(5,pi)       =  VARPHI_FPI;

%[7] Taylor rule
%i - VARPHI_R*r_nat - VARPHI_PI*pi - VARPHI_X*x = VARPHI_LI*i(-1) + VARPHI_LX*x(-1) + VARPHI_LPI*pi(-1) + VARPHI_FX*x(+1) + VARPHI_FPI*pi(+1) 
A0(6,i)        =  1;
A0(6,r_nat)    = -VARPHI_R;
A0(6,pi)       = -VARPHI_PI;
A0(6,x)        = -VARPHI_X;
A1(6,i)        =  VARPHI_LI;
A1(6,x)        =  VARPHI_LX;
A1(6,pi)       =  VARPHI_LPI;
A2(6,x)        =  VARPHI_FX;
A2(6,pi)       =  VARPHI_FPI;

% Initialise weight matrix:
W         = zeros(dy); % This is the weight matrix in the loss function
W(pi,pi)  = 1;
W(x,x)    = 1; %KAPPA/THETA;