function [sol] = fonc_FBL(index,level,y,p,r)
% Declare and solve linear system of equations
% A_0*[y_t^k; pi_t^k; i_t] = [Cout*Bout; Cinf*Binf]

global beta omega sigma phi_pi phi_y varphi zlb T

A0 = [1 0 1/sigma;
     -(1-beta*varphi)*(omega+inv(sigma)) (1+varphi/(1-varphi)-(1-beta*varphi)) 0;
     -phi_y -phi_pi 1];

%% Find RHS of output equation

out_discounting   = zeros(1,(T-index-1));
out_expectation   = zeros(T-index-1,1);
int               = zeros(T-index-1,1);

for s = 1:(T-index-1)
    out_discounting(s) = beta^s;
end

int                = r((index+1):(T-1),1)+phi_pi*p((index+1):(T-1),level-1)+phi_y*y((index+1):(T-1),level-1);
int                = max(int,zlb);
out_expectation(:) = (1-beta)*y((index+2):T,level-1)-(1/sigma)*(int-r((index+1):(T-1),1)-p((index+2):T,level-1));

% obtain RHS from output eqn and infl eqn
out_rhs = (1-beta)*y(index+1,level-1) - (1/sigma)*(-r(index,1)-p(index+1,level-1)) + out_discounting*out_expectation;

%% find RHS of inflation equation
% handle summation with C and B
infl_discounting = zeros(1,T-index-1);
infl_expectation = zeros(T-index-1,1);

for s = 1:(T-index-1)
    infl_discounting(s) = (1-beta*varphi)*(beta*varphi)^s;
end

infl_expectation(:) = p((index+1):(T-1),level-1) + (omega+inv(sigma))*y((index+1):(T-1),level-1);

% obtain RHS from output eqn and infl eqn
inf_rhs = varphi/(1-varphi)*p(index-1,level) + infl_discounting*infl_expectation;

tr_rhs = r(index,1);

%% solve system
% solve ignoring ZLB
RHS = [out_rhs;
       inf_rhs;
       tr_rhs];
   
sol    = inv(A0)*RHS;

% account for ZLB if necessary
if (sol(3) < zlb)
    A0(3,:) = [0 0 1];
    
    tr_rhs = zlb;
   
    RHS = [out_rhs;
           inf_rhs;
           tr_rhs];

    sol    = inv(A0)*RHS;
    
end

end