clear all; clc; close all;
% -------------------------------------------------------------------------
% Here we solve the forward looking equations with a fiscal multiplier with
% level-k thinking
% Note- code takes a long time to run. Output from code used in paper is
% saved as .mat file 
% -------------------------------------------------------------------------

% global paremeters 
beta = 0.99; 
varphi = 0.85;
kappa = (1-beta*varphi)*(1-varphi)/(varphi);
phi_pi = 1.5;
phi_y  = 0.5;

omegainv = 1/2;
sigma = 2;

svar = .2;
gbar = .01;
prob = 5/6; 

% important params
gpeg = 10000;  %number of quarters
k = 500;   %number of levels 

%k=3;

%initialize matrices
c = zeros(gpeg+1,k+1);   
p = zeros(gpeg+1,k+1);
y = zeros(gpeg+1,k+1);

% g pegged at gbar for gpeg periods then returns to 0
g = zeros(gpeg+1,1);

g(1:gpeg,1) = gbar;

% compute level k experiment
% loop through k levels - first column is level-0
for level = 2:k+1
   
   level
    
    for index = 1:gpeg
            
        % Initialize matrices
        c_discounting = zeros(1,gpeg-index+1);
        c_expectation = zeros(gpeg-index+1,1);
        infl_discounting = zeros(1,gpeg-index);
        temp_infl_prob = zeros(gpeg-index,1);
        temp_cons_prob = zeros(gpeg-index,1);
        infl_expectation = zeros(gpeg-index,1);
        
        for s = 0:(gpeg-index)
          c_discounting(s+1) = prob^(s+1)*beta^s;
        end
        c_expectation(:) = (1-beta)*c(index+1:gpeg+1,level-1) - 1/sigma*(0 - p(index+1:gpeg+1,level-1));
        
        c(index,level) = c_discounting*c_expectation;
    
        y(index,level) = (1-svar).*c(index,level) + svar.*g(index,1);       

        for s = 1:(gpeg-index)
            infl_discounting(s) = (beta*varphi)^s;
            temp_infl_prob(s) = prob^(s+1);
            temp_cons_prob(s) = prob^s;
        end
        
        infl_expectation(:) = temp_infl_prob.*beta*(1-varphi).*p(index+2:gpeg+1,level-1) + ...
            kappa*temp_cons_prob.*(sigma.*c(index+1:gpeg,level-1) + omegainv.*y(index+1:gpeg,level-1));
        
        p(index,level) = infl_discounting*infl_expectation + beta*(1-varphi).*prob*p(index+1,level-1) + ...
            kappa*(sigma.*c(index,level) + omegainv.*y(index,level));
        
    end
end

multiplier = y(1:gpeg,2:end)/(gbar*svar);  %calculates FM and removes level = 0 column

% % obtain result under RE
 UD = 0; % no ueber discounting
 IRF_all = FM_under_RE(gpeg,prob,UD,UD);
 multiplier_RE = IRF_all(1,:)/(svar);

% save values
 save(strcat('vars_t',num2str(gpeg),'_prob08333'));
save('CHECK');
    