function [y,p,r] = solve_OP(Tmax, shock_loc, num_levels, nr_shock, int_shock)
% -------------------------------------------------------------------------
% Solve system of equations contained in fonc_OP.m
% -------------------------------------------------------------------------

global beta sigma kappa varphi T damp

%initialize matrices
T = Tmax;  %number of quarters to run code to 
k = num_levels;   %number of levels 

%global paremeters 
beta = 0.99;
omega = 0;
sigma = 1;
kappa = 0.0255;
rho_nr = 0.9;

options  = optimset('Display','Off','TolFun',1e-12);
x_start  = .5;
varphi = fsolve(inline('P1 - (1-P2*x)*(1-x)/x*(P3 + inv(P4))',4),x_start,options,kappa,beta,omega,sigma);

y  = zeros(T+1,k+1);   
p  = zeros(T+1,k+1);
r  = zeros(T+1,k+1);

ybel = y;
pbel = p;

nr = zeros(T+1,1);

% create natural rate vector 
nr(1,1) = 0;
nr(2,1) = nr_shock;      % large shock appears at time T=1
for t = 3:T+1
    nr(t,1) = rho_nr*nr(t-1,1);
end
nr = nr(2:T+1,1);  % remove time T=0

% create interest rate shock vector with shock
m = zeros(T,1);
m(shock_loc,1) = int_shock;

% loop through k levels
for level = 2:k+1
    for index = 1:T 
        [sol] = fonc_OP(index,level,ybel,pbel,nr,m);

        y(index,level) = sol(1);
        p(index,level) = sol(2);
        r(index,level) = sol(3);

        ybel(index,level) = damp*y(index,level-1) + (1-damp)*sol(1);
        pbel(index,level) = damp*p(index,level-1) + (1-damp)*sol(2);
        
    end
end

end


    
    
    
    
    
    
    
    
    
    
    
    
    
    